#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "fitsdebug.h"
#include "utilities.h"

	
int fd_write_row(fd_file *file, ...) {

  /* write a new row of information to a fits table */

  int i, ncol = fd_get_ncol(file), status = 0;
  long firstrow = fd_get_nrow(file)+1, 
    firstelem = 1, 
    nelem = 1; 
  va_list ap;
  darray data;

  /* handle flush/opening of file */
  fd_flush_open(file);


  va_start(ap, file);
  for (i = 0; i < ncol; i++) {
    if (file->typecode[i].vla != 0)
      nelem = file->typecode[i].vla_length;
    else
      nelem = 1;

    /* allocate space */
    fd_darray_malloc(&data, file->typecode[i], nelem);
    /* read in value  */
    fd_darray_value_1row(&data, file->typecode[i], nelem, &ap); 

    /* write this row to the fitsfile */
    if (fd_darray_write_col(file, &data, file->typecode[i], 
			    i+1, firstrow, firstelem, nelem, &status))
      fd_err_message(status);


    /* free allocated memory */
    fd_darray_free(&data, file->typecode[i], nelem);
  }
  va_end(ap);    

  /* update the row value in the fd_file structure */
  fd_set_nrow(file,fd_get_nrow(file)+1);

  /* handle flush/closing of file */
  fd_flush_close(file);

  return 0;

}
	
int fd_vwrite_row(fd_file *file, va_list *args) {

  /* write a new row of information to a fits table */

  int i, ncol = fd_get_ncol(file), status = 0;
  long firstrow = fd_get_nrow(file)+1, 
    firstelem = 1, 
    nelem = 1; 
  darray data;


  /* handle flush/opening of file */
  fd_flush_open(file);


  for (i = 0; i < ncol; i++) {
    if (file->typecode[i].vla != 0)
      nelem = file->typecode[i].vla_length;
    else
      nelem = 1;

    /* allocate space */
    fd_darray_malloc(&data, file->typecode[i], nelem);
    /* read in value  */
    fd_darray_value_1row(&data, file->typecode[i], nelem, args); 

    /* write this row to the fitsfile */
    if (fd_darray_write_col(file, &data, file->typecode[i], 
			    i+1, firstrow, firstelem, nelem, &status))
      fd_err_message(status);


    /* free allocated memory */
    fd_darray_free(&data, file->typecode[i], nelem);
  }

  /* update the row value in the fd_file structure */
  fd_set_nrow(file,fd_get_nrow(file)+1);

  /* handle flush/closing of file */
  fd_flush_close(file);

  return 0;

}
	
void fd_darray_value_1row(darray *data, typecodeS typecode, 
			  long nelem, va_list *ap) {

  char *tmp;
  char  **tmpchr;
  short  *tmpsht;
  long   *tmplng;
  int    *tmpint;
  float  *tmpflt;
  double *tmpdbl;
  unsigned char *tmpbyt;
  int i;

  switch (typecode.int_type) {
  case TBIT: 
    msg_message(0,"fd_darray_value_1row: bit type not yet implemented\n");
    break;
  case TBYTE:
    if (nelem == 1) {
      data->byt_val[0] = (unsigned char) va_arg(*ap, int);
    } else {
      tmpbyt = (unsigned char *) va_arg(*ap, int *);
      for (i = 0; i < nelem; i++) 
	data->byt_val[i] = tmpbyt[i];
    }
    break;
  case TLOGICAL:
    msg_message(0,"fd_darray_value_1row: logical type not yet implemented\n");
    break;
  case TSTRING:  
    if (nelem == 1) {
      tmp = va_arg(*ap, char *);
      data->str_val[0] = mem_malloc_safe(typecode.size*sizeof(char));
      strcpy(data->str_val[0],tmp);
    } else {
      msg_message(0, "Variable Length Array not implememted for character strings\n");
      /*tmpchr = va_arg(*ap, char **); 
	for (i = 0; i < nelem; i++) {
	data->str_val[i] = mem_malloc_safe(typecode.size*sizeof(char));
	strcpy(data->str_val[i],tmpchr[i]); 
	}*/
    }
    break;
  case TSHORT:
    if (nelem == 1) {
      data->sht_val[0] = (short) va_arg(*ap, int);
    } else {
      tmpsht = (short *) va_arg(*ap, int *);
      for (i = 0; i < nelem; i++) 
	data->sht_val[i] = tmpsht[i];
    }
    break;
  case TINT:
    if (nelem == 1) {
      data->int_val[0] = va_arg(*ap, int); 
    } else {
      tmpint = va_arg(*ap, int *);
      for (i = 0; i < nelem; i++) 
	data->int_val[i] = tmpint[i];
    }
    break;
  case TLONG:
    if (nelem == 1) {
      data->lng_val[0] = va_arg(*ap, long);
    } else {
      tmplng = va_arg(*ap, long *); 
      for (i = 0; i < nelem; i++) 
	data->lng_val[i] = tmplng[i];
    }
    break;
  case TFLOAT:
    if (nelem == 1) {
      data->flt_val[0] = (float) va_arg(*ap, double);
    } else {
      tmpflt = (float *) va_arg(*ap, double *); 
      for (i = 0; i < nelem; i++) 
	data->flt_val[i] = tmpflt[i];
    }
    break;
  case TDOUBLE:
    if (nelem == 1) {
      data->dbl_val[0] = va_arg(*ap, double);
    } else {
      tmpdbl = va_arg(*ap, double *);
      for (i = 0; i < nelem; i++) 
	data->dbl_val[i] = tmpdbl[i];
    }
    break;
  case TCOMPLEX:
    msg_message(0,"fd_darray_value_1row: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_darray_value_1row: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_write_value_1row: unable to extract type information\n");
    break;
  }

}

	
int fd_write_rows(fd_file *file, long nrows, ...) {

  /* write a new row of information to a fits table */

  int i, row, ncol = fd_get_ncol(file), status = 0;
  long firstrow = fd_get_nrow(file)+1, 
    firstelem = 1, 
    nelem = nrows; 
  va_list ap;
  darray data;

  /* handle flush/opening of file */
  fd_flush_open(file);

  va_start(ap, nrows);

  for (i = 0; i < ncol; i++) {

    if (file->typecode[i].vla) {

      for (row = 0; row < nrows; row++) {

	nelem = file->typecode[i].vla_length;
	/* allocate space */
	fd_darray_malloc(&data, file->typecode[i], nelem);
	/* read in value  */
	fd_darray_value_nrow(&data, file->typecode[i], nelem, &ap); 
	
	/* write this row to the fitsfile */
	if (fd_darray_write_col(file, &data, file->typecode[i], 
				i+1, firstrow+row, firstelem, nelem, &status))
	  fd_err_message(status);
      
      
	/* free allocated memory */
	fd_darray_free(&data, file->typecode[i], nelem);

      }
    } else {
      /* allocate space */
      fd_darray_malloc(&data, file->typecode[i], nelem);
      /* read in value  */
      fd_darray_value_nrow(&data, file->typecode[i], nelem, &ap); 
      /* fd_darray_print(&data, file->typecode[i], nelem); */
      
      /* write this row to the fitsfile */
      if (fd_darray_write_col(file, &data, file->typecode[i], 
			      i+1, firstrow, firstelem, nelem, &status))
	fd_err_message(status);
      
      
      /* free allocated memory */
      fd_darray_free(&data, file->typecode[i], nelem);
    }
  }
  va_end(ap);    

  /* update the row value in the fd_file structure */
  fd_set_nrow(file,fd_get_nrow(file)+nrows);

  /* handle flush/closing of file */
  fd_flush_close(file);

  return 0;

}
	
void fd_darray_value_nrow(darray *data, typecodeS typecode, 
			  long nelem, va_list *ap) {

  char  **tmpchr;
  short  *tmpsht;
  long   *tmplng;
  int    *tmpint;
  float  *tmpflt;
  double *tmpdbl;
  unsigned char *tmpbyt;
  int i;
  int size = typecode.size;


  switch (typecode.int_type) {
  case TBIT: 
    msg_message(0,"fd_darray_value_nrow: bit type not yet implemented\n");
    break;
  case TBYTE:
    tmpbyt = (unsigned char *) va_arg(*ap, int *);
    for (i = 0; i < nelem; i++) 
      data->byt_val[i] = tmpbyt[i];
    break;
  case TLOGICAL:
    msg_message(0,"fd_darray_value_nrow: logical type not yet implemented\n");
    break;
  case TSTRING:
    if (typecode.vla != 0) {
      msg_message(0, "Variable Length Array not implememted for character strings\n");
    }
    tmpchr = va_arg(*ap, char **); 
    for (i = 0; i < nelem; i++) {
      data->str_val[i] = mem_malloc_safe(size*sizeof(char));
      strcpy(data->str_val[i],tmpchr[i]); 
    }
    break;
  case TSHORT:
    tmpsht = (short *) va_arg(*ap, int *);
    for (i = 0; i < nelem; i++) 
      data->sht_val[i] = tmpsht[i];
    break;
  case TINT:
    tmpint = va_arg(*ap, int *);
    for (i = 0; i < nelem; i++) 
      data->int_val[i] = tmpint[i];
    break;
  case TLONG:
    tmplng = va_arg(*ap, long *); 
    for (i = 0; i < nelem; i++) 
      data->lng_val[i] = tmplng[i];
    break;
  case TFLOAT:
    tmpflt = (float *) va_arg(*ap, double *); 
    for (i = 0; i < nelem; i++) 
      data->flt_val[i] = tmpflt[i];
    break;
  case TDOUBLE:
    tmpdbl = va_arg(*ap, double *);
    for (i = 0; i < nelem; i++) 
      data->dbl_val[i] = tmpdbl[i];
    break;
  case TCOMPLEX:
    msg_message(0,"fd_darray_value_nrow: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_darray_value_nrow: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_write_value_nrow: unable to extract type information\n");
    break;
  }

}

int fd_darray_write_col(fd_file *file, darray *data, typecodeS typecode, 
			int col, long firstrow, long firstelem,
			long nelem, int *status) {

  switch (typecode.int_type) {
  case TBIT: 
    msg_message(0,"fd_darray_write_col: bit type not yet implemented\n");
    break;
  case TBYTE:
    if (fits_write_col_byt(file->fptr, col, firstrow,
			   firstelem, nelem, data->byt_val, status))
      return *status;
    break;
  case TLOGICAL:
    msg_message(0,"fd_darray_write_col: logical type not yet implemented");
    break;
  case TSTRING:  
    if (fits_write_col_str(file->fptr, col, firstrow, 
			   firstelem, nelem, data->str_val, status))
      return *status;
    break;
  case TSHORT:
    if (fits_write_col_sht(file->fptr, col, firstrow, 
			   firstelem, nelem, data->sht_val, status))
      return *status;
    break;
  case TINT:
    if (fits_write_col_int(file->fptr, col, firstrow, 
			   firstelem, nelem, data->int_val, status))
      return *status;
    break;
  case TLONG:
    if (fits_write_col_lng(file->fptr, col, firstrow, 
			   firstelem, nelem, data->lng_val, status))
      return *status;
    break;
  case TFLOAT:
    if (fits_write_col_flt(file->fptr, col, firstrow, 
			   firstelem, nelem, data->flt_val, status))
      return *status;
    break;
  case TDOUBLE:
    if (fits_write_col_dbl(file->fptr, col, firstrow, 
			   firstelem, nelem, data->dbl_val, status))
      return *status;
    break;
  case TCOMPLEX:
    msg_message(0,"fd_darray_write_col: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_darray_write_col: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_write_write_col: unable to extract type information\n");
    break;
  }

  return 0;

}

void fd_darray_print(darray *data, typecodeS typecode, long nelem) {

  int i;

  switch (typecode.int_type) {
  case TBIT: 
    msg_message(0,"fd_darray_print: bit type not yet implemented\n");
    break;
  case TBYTE:
    for (i = 0; i < nelem; i++)
      printf((i==0)?"-- %u\n":"   %u\n",data->byt_val[i]); 
    break;
  case TLOGICAL:
    msg_message(0,"fd_darray_print: logical type not yet implemented\n");
    break;
  case TSTRING: 
    for (i = 0; i < nelem; i++)
    printf((i==0)?"-- %s\n":"   %s\n",data->str_val[i]);
    break;
  case TSHORT:
    for (i = 0; i < nelem; i++)
    printf((i==0)?"-- %d\n":"   %d\n",data->sht_val[i]);
    break;
  case TINT:
    for (i = 0; i < nelem; i++)
    printf((i==0)?"-- %d\n":"   %d\n",data->int_val[i]);
    break;
  case TLONG:
    for (i = 0; i < nelem; i++)
    printf((i==0)?"-- %d\n":"   %d\n",data->lng_val[i]);
    break;
  case TFLOAT:
    for (i = 0; i < nelem; i++)
    printf((i==0)?"-- %f\n":"   %f\n",data->flt_val[i]);
    break;
  case TDOUBLE:
    for (i = 0; i < nelem; i++)
    printf((i==0)?"-- %12.10f\n":"   %12.10f\n",data->dbl_val[i]);
    break;
  case TCOMPLEX:
    msg_message(0,"fd_darray_print: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_darray_print: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_write_print: unable to extract type information\n");
    break;
  }


}

void fd_darray_malloc(darray *data, typecodeS typecode, long nelem) {

  switch (typecode.int_type) {
  case TBIT: 
    msg_message(0,"fd_darray_malloc: bit type not yet implemented\n");
    break;
  case TBYTE:
    data->byt_val = mem_malloc_safe(nelem*sizeof(unsigned char));
    break;
  case TLOGICAL:
    msg_message(0,"fd_darray_malloc: logical type not yet implemented\n");
    break;
  case TSTRING: 
    data->str_val = mem_malloc_safe(nelem*sizeof(char *));
    break;
  case TSHORT:
    data->sht_val = mem_malloc_safe(nelem*sizeof(short));
    break;
  case TINT:
    data->int_val = mem_malloc_safe(nelem*sizeof(int));
    break;
  case TLONG:
    data->lng_val = mem_malloc_safe(nelem*sizeof(long));
    break;
  case TFLOAT:
    data->flt_val = mem_malloc_safe(nelem*sizeof(float));
    break;
  case TDOUBLE:
    data->dbl_val = mem_malloc_safe(nelem*sizeof(double));
    break;
  case TCOMPLEX:
    msg_message(0,"fd_darray_malloc: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_darray_malloc: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_write_malloc: unable to extract type information\n");
    break;
  }

}
	 
void fd_darray_free(darray *data, typecodeS typecode, long nelem) {

  int i = 0;
  
  switch (typecode.int_type) {
  case TBIT: 
    msg_message(0,"fd_darray_free: bit type not yet implemented\n");
    break;
  case TBYTE:
    free(data->byt_val);
    break;
  case TLOGICAL:
    msg_message(0,"fd_darray_free: logical type not yet implemented\n");
    break;
  case TSTRING: 
    for (i = 0; i < nelem; i++)
      free(data->str_val[i]);
    free(data->str_val);
    break;
  case TSHORT:
    free(data->sht_val);
    break;
  case TINT:
    free(data->int_val);
    break;
  case TLONG:
    free(data->lng_val);
    break;
  case TFLOAT:
    free(data->flt_val);
    break;
  case TDOUBLE:
    free(data->dbl_val);
    break;
  case TCOMPLEX:
    msg_message(0,"fd_darray_free: complex type not yet implemented\n");
    break;
  case TDBLCOMPLEX:
    msg_message(0,"fd_darray_free: double complex type not yet implemented\n");
    break;
  default:
    msg_exit(2,"fd_write_free: unable to extract type information\n");
    break;
  }

}
