#include <stdlib.h>
#include <stdio.h>
#include <math.h>

double cheby_series(double x, double *cs, int n) {

/* 
   c     april 1977 version.  w. fullerton, c3, los alamos scientific lab.
   c
   c evaluate the n-term chebyshev series cs at x.  adapted from
   c r. broucke, algorithm 446, c.a.c.m., 16, 254 (1973).  also see fox
   c and parker, chebyshev polys in numerical analysis, oxford press, p.56.
   c
   c             input arguments --
   c x      value at which the series is to be evaluated.
   c cs     array of n terms of a chebyshev series.  in eval-
   c        uating cs, only half the first coef is summed.
   c n      number of terms in array cs.
   c
   
   Ported to C, 10-DEC-97 by Randall Smith, SAO 
   */

  double b2=0., b1=0., b0=0., twox;
  int i, ni;

  if ((n < 1)||(n > 1000)) {
    printf("cheby_series: Illegal Number of terms = %d\n",n);
    abort();
  }
  
  if ((x < -1.1)||(x > 1.1)) {
    printf("cheby_series: x=%e outside legal range of -1,+1\n",x);
  }
  
  twox = 2.*x;
  for (i=0;i<n;i++) {
    b2 = b1;
    b1 = b0;
    ni = n + 1 - i;
    b0 = twox*b1 - b2 + cs[ni];
  }

  return 0.5 * (b0-b2);
}
