#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#define TRUE 1
#define FALSE 0

double interpol_huntd(int n, double *x, double *y, double z) {

  double grad,d1,df,f;
  int inc, jl, jm, ju;
  
  inc = (x[n-1] > x[0]) ? TRUE : FALSE;
  if (( inc && (z < x[0] || z > x[n-1])) ||
      (!inc && (z > x[0] || z < x[n-1]))) {
    printf("interpol_huntd: Asking for %e, min is %e, max is %e\n",z,
	    x[0],x[n-1]);
    printf("interpol_huntd: Cannot extrapolate\n");
    abort();
  }  
  jl = 0;
  ju = n;
  while (ju - jl > 1) {
    jm = (ju + jl) / 2;
    if ((z > x[jm]) == inc) {
      jl = jm;
    } else {
      ju = jm;
    }
  }
  /* ------	Z is sandwiched between JL and JU ------ */
  if ((x[jl] > 0. && x[ju] > 0.) && (y[jl] > 0. && y[ju] > 0.)) {
    grad = (log10(y[ju]) - log10(y[jl])) /
      (log10(x[ju]) - log10(x[jl]));
    df = grad * (log10(z) - log10(x[jl]));
    d1 = log10(y[jl]) + df;
    f = pow(10., d1);
  } else {
    f = y[jl]+(y[ju]-y[jl])*((z-x[jl])/(x[ju]-x[jl]));
  }
  return f;
}

float interpol_huntf(int n, float *x, float *y, float z) {

  float grad,d1,df,f;
  int inc, jl, jm, ju;
  
  inc = (x[n-1] > x[0]) ? TRUE : FALSE;
  if (( inc && (z < x[0] || z > x[n-1])) ||
      (!inc && (z > x[0] || z < x[n-1]))) {
    printf("interpol_huntf: Asking for %e, min is %e, max is %e\n",z,
	    x[0],x[n-1]);
    printf("interpol_huntf: Cannot extrapolate\n");
    abort();
  }  
  jl = 0;
  ju = n;
  while (ju - jl > 1) {
    jm = (ju + jl) / 2;
    if ((z > x[jm]) == inc) {
      jl = jm;
    } else {
      ju = jm;
    }
  }
  /* ------	Z is sandwiched between JL and JU ------ */
  if ((x[jl] > 0. && x[ju] > 0.) && (y[jl] > 0. && y[ju] > 0.)) {
    grad = (log10(y[ju]) - log10(y[jl])) /
      (log10(x[ju]) - log10(x[jl]));
    df = grad * (log10(z) - log10(x[jl]));
    d1 = log10(y[jl]) + df;
    f = pow(10., d1);
  } else {
    f = y[jl]+(y[ju]-y[jl])*((z-x[jl])/(x[ju]-x[jl]));
  }
  return f;
}

