
/*! \file calc_maxwell_rates.h 
 * \brief Collisional excitation data types                 
 * Data are stored in a number of different formats; 
 * this header lists all the currently-used versions 
 * which are found in calc_maxwell_rates.c 
 */
#ifndef MAX_UPS
#define MAX_UPS 20 /*!< Length of temp/om arrays in collision structure */
#endif

#ifndef MAX_CHI
#define MAX_CHI 200. /*!< Do not calculate rate if kT/dE greater than this.*/
#endif

#ifndef KBOLTZ
#define KBOLTZ 8.617385e-8  /*!< in units of keV/K */
#endif

#ifndef M_E
#define M_E 2.7182818284590452354  /*!< Euler e */
#endif

#ifndef UPSILON_COLL_COEFF 
#define UPSILON_COLL_COEFF 8.629e-6  /*!< sqrt{2 pi / kB} hbar^2/m_e^{1.5} */
#endif

#define E_UPSILON 1    /*!< Electron upsilon values (unitless) */
#define E_RATE_COEFF 2 /*!< Electron rate coefficient (cm^3/s) */
#define P_UPSILON 3    /*!< Proton upsilon values (unitless) */
#define P_RATE_COEFF 4 /*!< Proton rate coefficient (cm^3/s) */

#define BURGESSTULLY 1 /*!< Burgess-Tully-type data*/
#define CHIANTI_1 11   /*!< CHIANTI pre-4.0 type 1 data (5 pt spline) */
#define CHIANTI_2 12   /*!< CHIANTI pre-4.0 type 2 data (5 pt spline) */
#define CHIANTI_3 13   /*!< CHIANTI pre-4.0 type 3 data (5 pt spline) */
#define CHIANTI_4 14   /*!< CHIANTI pre-4.0 type 4 data (5 pt spline) */
#define CHIANTI_5 15   /*!< CHIANTI pre-4.0 type 5 data (5 pt spline) */
#define CHIANTI_6 16   /*!< CHIANTI pre-4.0 type 6 data (5 pt spline) */

#define CHIANTI4_1 21   /*!< CHIANTI 4.0 type 1 data (9 pt spline) */
#define CHIANTI4_2 22   /*!< CHIANTI 4.0 type 2 data (9 pt spline) */
#define CHIANTI4_3 23   /*!< CHIANTI 4.0 type 3 data (9 pt spline) */
#define CHIANTI4_4 24   /*!< CHIANTI 4.0 type 4 data (9 pt spline) */
#define CHIANTI4_5 25   /*!< CHIANTI 4.0 type 5 data (9 pt spline) */
#define CHIANTI4_6 26   /*!< CHIANTI 4.0 type 6 data (9 pt spline) */

#define SGC_1 31   /*!< Sampson, Goett and Clark (1983) S-type He-like data */
#define SGC_2 32   /*!< Sampson, Goett and Clark (1983) P-type He-like data */
#define SGC_3 33   /*!< Sampson, Goett and Clark (1983) S-type H-like data */
#define KATO_NAKAZAKI_1 41  /*!< Kato and Nakazaki (1989), ADNDT 42, 313 */
#define KATO_NAKAZAKI_2 42  /*!< Kato and Nakazaki (1989), ADNDT 42, 313 */
/* These must be spaced by at least MAX_UPS */
#define INTERP_E_UPSILON 100     /*!< Include both left & right boundaries */
#define INTERP_P_UPSILON 200     /*!< Include both left & right boundaries */
#define INTERP_E_RATE_COEFF 300  /*!< Include both left & right boundaries */
#define INTERP_P_RATE_COEFF 400  /*!< Include both left & right boundaries */

#define INTERP_E_UPS_OPEN 150    /*!< Include neither boundary */
#define INTERP_P_UPS_OPEN 250    /*!< Include neither boundary */
#define INTERP_E_RATE_OPEN 350   /*!< Include neither boundary */
#define INTERP_P_RATE_OPEN 450   /*!< Include neither boundary */

#define INTERP_E_UPS_INC_MIN 500   /*!< Include only minimum; max is out */
#define INTERP_P_UPS_INC_MIN 600   /*!< Include only minimum; max is out */
#define INTERP_E_RATE_INC_MIN 700  /*!< Include only minimum; max is out */
#define INTERP_P_RATE_INC_MIN 800  /*!< Include only minimum; max is out */

#define INTERP_E_UPS_INC_MAX 550   /*!< Include only maximum; min is out */
#define INTERP_P_UPS_INC_MAX 650   /*!< Include only maximum; min is out */
#define INTERP_E_RATE_INC_MAX 750  /*!< Include only maximum; min is out */
#define INTERP_P_RATE_INC_MAX 850  /*!< Include only maximum; min is out */

#define PROTON_BT 1001 /*!< For Burgess-Tully Proton excitation rates */
