#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "messages.h"

#ifndef KBOLTZ
#define KBOLTZ 8.617385e-8  /* kB = [keV/K] */
#endif

double exint_n(double x, double E1x_in, int n);

double calc_sampson_s(double om[], int Z, double Te) {

  /* These routines come from Sampson, Goett, & Clark, ADNDT 29, 467 */
  
  double result=0.0;
  double kT, y;
  double dE, z2s_h, a1_gam, a1e_gam, a2, c0, c1, c2;
  double a2e, cere, cere1, s, se;
  double a1, a1y, E1y, Ery, Er1y;
  double Zeff, Zeff_e, term;
  int re;
  double Z2gamma=0.0, Z2gamma_e=0.0;

  dE     = om[0];
  a1_gam = om[1];
  a1e_gam= om[2];
  z2s_h  = om[3];
  a2     = om[4];
  c0     = om[5];
  c1     = om[6];
  c2     = om[7];
  a2e    = om[8];
  cere   = om[9];
  cere1  = om[10];
  re     = (int) om[11];
  s      = om[12];
  se     = om[13];
  
  kT = KBOLTZ*Te;
  y = dE/kT;

  a1 = a2+1.0;
  a1y = a1*y;
  E1y = exint_n(y,-1.,1);
  Ery = exint_n(a1y,-1.,1);
  Er1y= exint_n(a1y,Ery,2);
  
  term = (c1*Ery + c2*Er1y/a1);
  if ((a1_gam !=0.0)&&(term > 0)) {
    Z2gamma = c0 + 1.333*z2s_h*E1y*exp(y) + y*exp(a1y)*term;
  } else { /* Avoid doing exponential */
    Z2gamma = c0 + 1.333*z2s_h*E1y*exp(y);
  }

  a1 = a2e+1;
  a1y = a1*y;
  Ery = exint_n(a1y,-1.,re);
  Er1y = exint_n(a1y,-1.,re+1);

  term = (cere*Ery/(pow(a1,re-1)) + cere1*Er1y/(pow(a1,re)));
  if ((a1e_gam != 0.0) && (term > 0)) {
    Z2gamma_e = y*exp(a1y)*term;
  } else {
    Z2gamma_e = 0.0;
  }

  Zeff = Z - s;
  Zeff_e = Z - se;
  
  result = a1_gam*Z2gamma/(Zeff*Zeff) + a1e_gam*Z2gamma_e/(Zeff_e*Zeff_e);

  return result;
}

double calc_sampson_p(double om[], int Z, double Te) {

  double result=0.0;
  double kT, y;
  double dE, a, z2s, c0, cr, cr1, r, s;
  double Z2gamma, Zeff, a1, a1y, E1y, Ery, Er1y;
  double term;

  dE  = om[0];
  a   = om[1];
  z2s = om[2];
  c0  = om[3];
  cr  = om[4];
  cr1 = om[5];
  r   = (int) om[6];
  s   = om[7];

  kT = KBOLTZ*Te;
  y  = dE/kT;

  a1 = a+1;
  a1y = a1*y;

  E1y = exint_n(y,-1.,1);
  Ery = exint_n(a1y,-1.,r);
  Er1y= exint_n(a1y,-1.,r+1);

  term = (cr*Ery/pow(a1,r-1) + cr1*Er1y/pow(a1,r));
  if (term > 0) {
    Z2gamma = c0 + 1.333*z2s*E1y*exp(y) + y*exp(a1y)*term;
  } else {
    Z2gamma = c0 + 1.333*z2s*E1y*exp(y);
  }

  Zeff = Z - s;
    
  result = Z2gamma / (Zeff*Zeff);

  return result;
}


double calc_sampson_h(double om[], int Z, double Te) {

  double result=0.0;
  double kT, y;
  double dE, z2s, a, c0,c1,c2,c_sw;
  double a1, a1y, E1y, Ery, Er1y, Zeff2;
  double term;

  dE   = om[0];
  z2s  = om[1];
  a    = om[2];
  c0   = om[3];
  c1   = om[4];
  c2   = om[5];
  c_sw = om[6];
  Zeff2 = (double) Z*Z; 

  kT  = KBOLTZ*Te;
  y   = dE/kT;
  a1  = a+1;
  a1y = a1*y;
  E1y = exint_n(y,-1.,1);
  Ery = exint_n(a1y,-1.,1);
  Er1y = exint_n(a1y,Ery,2);  /* This is E_2(a1y) */

  term = (c1*Ery + c2*Er1y/a1);
  if (term > 0) {
    result = c0 + 1.333*z2s*E1y*exp(y) + y*exp(a1y)*term;
  } else {
    result = c0 + 1.333*z2s*E1y*exp(y);
  }

  result *= 2*c_sw/Zeff2;
  
  return result;
}
