#include <stdlib.h>

#include "apec.h"
#include "messages.h"
#define BURGESSTULLY 1
#define NO_COLL_DATA -1
#define TRUE 1
#define FALSE 0

void calc_ioniz_rate(double Te, double Ne, struct DEBUG_FILES *debug,
		     struct PARAMETERS *params,
		     struct ELEMENT_ION_STATE *state, double *ioniz_rate) {

  int ish;
  int skip;

  /* RKS: Coefficients for ionization rate fits, from */
  /* Younger, S. M.  1982, JQSRT, 27, 541.   Does not use earlier version */
  /* (which is similar) from Younger, S. M.  1981, JQSRT, 26, 329. */

  double a=0.,b=0.,c=0.,d=0.;  
                   /* NSB: coefficients are in units of 1.0e-14 cm3/sec */
                   /* NSB: Or just change to call exint1.c */
  /* RKS: If these are similar to Arnaud & Raymond coeffs, they should have */
  /* units of 10^-14 cm^2 eV^2.  */

  double E_IP=0.;    /* ionization potential energy */
  double ch;
  double chi;     /* reduced temperature, Te/(BOLTZ*E_IP) */
  double chi2;
  double chi3;
  double alpha;
  double beta;
  double fchi;    /* distorted wave factor */
  double q;       

  q = 0.;
  skip = FALSE;  /* By default, should have data. */

  for (ish = 0; ish < state->ion_inf->ns; ish++) {
    switch (state->ionrec_dat[ish].colltype) {
    case (NO_COLL_DATA):
      skip = TRUE;
      break;
    case (BURGESSTULLY):
      skip = FALSE;
      a = state->ionrec_dat[ish].cc[0]; 
      b = state->ionrec_dat[ish].cc[1]; 
      c = state->ionrec_dat[ish].cc[2]; 
      d = state->ionrec_dat[ish].cc[3]; 
      E_IP = state->ionrec_dat[ish].E; 
      break;
    default:
      errmess("get_ioniz_recomb","Illegal collision type");
      break;
    }
    
    if (!skip) {
      if (E_IP > 0) {
	chi = Te / (BOLTZ * E_IP);  
      } else {
	chi = (Te / BOLTZ)*BLOWUP;
      }
      chi2 = chi * chi ; 
      chi3 = chi * chi2;
      
      /* NSB: John's alpha was from 1982 paper */
      /* RKS: Checked.  This is from Younger, S. M.  1982, JQSRT, 27, 541 */
      
      alpha = (.001193 + .9764*chi + .6604*chi2 + .02590*chi3) /
	(1.0 + 1.488*chi + .2972*chi2 + .004925*chi3) ;
      
      /* NSB: John's beta was from 1981 paper, but I changed it to 1982? */
      /* RKS: This is indeed 1982; however, earlier version, as well as */
      /* RS code, had an error in the chi3 term; it was 0.002503.  The */
      /* current value, 0.002053, is correct. */
      
      beta = (-.0005725 + .01345*chi + .8691*chi2 + .03404*chi3) /
	(1.0 + 2.197*chi + .2454*chi2 + .002053*chi3) ;
      
      if (debug->do_qioniz && debug->ascii) {
	fprintf(debug->qioniz.ascii,"%e %e %e\n", 
		chi, alpha, beta);
      }

      ch = 1. / chi;
      fchi = 0.3*ch*(a+b*(1.+ch) + (c-(a+b*(2.+ch))*ch)*alpha+ d * beta * ch) ;
      
      /* RKS: fchi has units of 1.e-14.  See collisional ionization */
      /* notes of 2/17/98. */
      
      if (debug->do_qioniz && debug->ascii) 
	fprintf(debug->qioniz.ascii,"  %e\n", fchi);

      if (debug->do_qioniz && debug->fits) {
	fd_write_row(&debug->qioniz.fits, chi, alpha, beta, fchi, 0.0);
      }
      
      q += 2.2e-6 * sqrt(chi) * fchi * exp(-ch) / (E_IP*sqrt(E_IP));
    } /* end of skip */
  }
  if (debug->do_qioniz && debug->ascii) {
    fprintf(debug->qioniz.ascii,"ioniz_rate = %g \n",q);
    fflush(debug->qioniz.ascii);
  }

  if (debug->do_qioniz && debug->fits) {
    fd_write_row(&debug->qioniz.fits, 0.0, 0.0, 0.0, 0.0, q);
  }

  /* qion has units of 1.e-14 cm^3 / s; cm^3 / s from the Seaton formula, */
  /* and 10^-14 from the distorted wave factor. */

  *ioniz_rate = 1.e-14*q;
}
