#include <stdlib.h>
#include <strings.h>
#include <stdio.h>

#include "messages.h"
#include "apec.h"

struct atomstr {
  int Z;
  int rmJ;
  char File[MAXSTRLEN];
  struct atomstr *next;
};

void extract_atom_input(struct PARAMETERS *params, struct atomstr *atoms) {

  /* Major improvement in code here.  In the case where an entire atom is
     passed to the code, all the ions are set up.  If individual ion states
     are given, a single state for each partial atom is created, filling in 
     only those ions requested. */

  int Z, NZ, iEl, i, match_rmJ;
  struct atomstr *ptr;

  int Z_request[MAXIMUM_Z+1];

  if (atoms==NULL) {
    errmess("extract_atom_input","No atomic information (SATOM) set.");
  } else {
    /* First, find out how many UNIQUE atoms have been input. */
    for (Z=1;Z<MAXIMUM_Z+1;Z++) Z_request[Z] = 0;
    ptr = atoms;
    while (ptr != NULL) {
      Z_request[ptr->Z] = 1;
      ptr = ptr->next;
    }
    NZ = 0;
    for (Z=1;Z<MAXIMUM_Z+1;Z++) if (Z_request[Z] == 1) NZ++;
    
    /* OK, so we've got NZ atoms.  How about ions? */
    params->N_Elements = NZ;
    params->list_Z    = (int *) malloc_safe(NZ*sizeof(int));
    params->list_NrmJ = (int *) malloc_safe(NZ*sizeof(int));
    
    iEl = 0;
    for (Z=1;Z<MAXIMUM_Z+1;Z++) {
      if (Z_request[Z] == 1) {
	params->list_Z[iEl] = Z;
	params->list_NrmJ[iEl] = 0;
	params->list_rmJ[iEl] = (int *) malloc_safe((Z+1)*sizeof(int));
	ptr = atoms;
	while (ptr != NULL) {
	  if (ptr->Z == Z) {
	    if (ptr->rmJ == -1) {
	      params->list_NrmJ[iEl] = Z+1;
	      for(i=0;i<Z+1;i++) params->list_rmJ[iEl][i] = i+1;
	    } else {
	      match_rmJ = 0;
	      for(i=0;i<params->list_NrmJ[iEl];i++) {
		if (ptr->rmJ == params->list_rmJ[iEl][i]) match_rmJ=1;
	      } 
	      if (!match_rmJ) {
		params->list_rmJ[iEl][params->list_NrmJ[iEl]] = ptr->rmJ;
		params->list_NrmJ[iEl]++;
	      }
	    }
	  }
	  ptr = ptr->next;
	} /* while (ptr != NULL) */
	iEl++;
      } /* if (Z_request[Z] == 1) */
    } /* for (Z=1 ... */
    
    /* So now we can clean up atoms */
    ptr = atoms;
    while (ptr != NULL) {
      atoms = ptr;
      ptr = ptr->next;
      free(atoms);
    }
  }
}
