#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "apec.h"
#include "messages.h"

void find_ion_bal(int Z, double Te, double Ne, double Ion_pop[], 
		  struct ION_BAL_TABLE *ionbal, struct PARAMETERS *params) {

  int i,iIonState, NT, ND;
  int iTlow=-1, iThigh=-1, iNlow=-1, iNhigh=-1;
  int indx;
  double w_Tlow=0., w_Thigh=0., w_Nlow=0., w_Nhigh=0.;
  double dT, dN;
  double a1,a2,a3,a4;

  NT = ionbal->Ntemp;
  ND = ionbal->Ndens;
  /* message("find_ion_bal","NT = %d, ND = %d",NT, ND); */
  
  /* Now, find where we are...*/
  for (i=0;i<NT;i++) {  /* Assumes T is monotonic increasing */
    /* message("find_ion_bal","ionbal->T[i] = %e",ionbal->T[i]); */
    if (((ionbal->T)[i] > Te)&&(iTlow==-1)) {
      if (i==0) {
	errmess("find_ion_bal","Te=%e out of range for table, min val=%e.",
		Te, (ionbal->T)[i]);
      } else {
	iTlow = i-1;
	iThigh = i;
	dT = (ionbal->T)[iThigh] - (ionbal->T)[iTlow];
	w_Tlow = 1 - (Te - (ionbal->T)[iTlow])/dT;
	w_Thigh= 1 - w_Tlow;
      }
    }
  }
  if (iTlow==-1) errmess("find_ion_bal","Te=%e out of range of table.",Te);

  if (ND > 1) {
    for (i=0;i<ND;i++) {  /* Assumes N is monotonic increasing */
      if (((ionbal->dens)[i] > Ne)&&(iNlow==-1)) {
	if (i==0) {
	  errmess("find_ion_bal","Ne=%e out of range for table, min val=%e",
		  Ne,(ionbal->dens)[i]);
	} else {
	  iNlow = i-1;
	  iNhigh = i;
	  dN = (ionbal->dens)[iThigh] - (ionbal->dens)[iNlow];
	  w_Nlow = 1 - (Ne - (ionbal->dens)[iNlow])/dN;
	  w_Nhigh= 1 - w_Nlow;
	}
      }
    }
  } else {
    /* message("find_ion_bal","No density diagnostics"); */
    iNlow = 0;
    iNhigh = 0;
    w_Nlow = 1.;
    w_Nhigh = 0.;
  }
  
  /* message("find_ion_bal","Te=%e,iTlow=%d,w_Tlow=%e,w_Thigh=%e, indx=%d",
     Te,iTlow,w_Tlow,w_Thigh,ionbal->indx[Z]); */

  for(iIonState=0;iIonState<(Z+1);iIonState++) {
    indx = ionbal->indx[Z] + iIonState;
    a1 = (ionbal->X[iTlow*ND +iNlow])[indx];
    a2 = (ionbal->X[iThigh*ND+iNlow])[indx];
    a3 = (ionbal->X[iTlow*ND +iNhigh])[indx];
    a4 = (ionbal->X[iThigh*ND+iNhigh])[indx];
    if ((a1>0)&&(a2>0)&&(a3>0)&(a4>0)) {
      a1 = log10(a1); a2 = log10(a2); a3 = log10(a3); a4 = log10(a4);
      Ion_pop[iIonState] = pow(10.,(a1*w_Tlow*w_Nlow + a2*w_Thigh*w_Nlow +
				    a3*w_Tlow*w_Nhigh+ a4*w_Thigh*w_Nhigh));
    } else {
      Ion_pop[iIonState] = a1*w_Tlow*w_Nlow + a2*w_Thigh*w_Nlow +
	a3*w_Tlow*w_Nhigh+ a4*w_Thigh*w_Nhigh;
    }
    
    if (Ion_pop[iIonState] == 0) {
      Ion_pop[iIonState] = 1/BLOWUP;
    }
    /* message("find_ion_bal","Z = %d, Ion = %d, Pop = %e",Z, iIonState,
       Ion_pop[iIonState]); */
  }

}
