#include <stdlib.h>
#include <stdio.h>

#include "messages.h"
#include "apec.h"
#include "datatypes.h"

struct ELEMENT_ION_STATE * get_state(int Z);
char *roman_element(int Z, int J);
void load_filemap(char *filemap_file, struct FILEMAP **filemap);
void read_ion_data(int require_checksum, int Z, int rmJ, 
		   struct FILEMAP *filemap, int *maxNlev,
		   struct ELEMENT_ION_STATE *state);

void get_atomic_data(char *filemap_file, int require_checksum, 
		     int N_Elements, int *list_Z, 
		     int *list_NrmJ, int *list_rmJ[], 
		     int *maxZ, int *maxNlev,
		     ElementP **Atomic_params) {

  int iZ, irmJ, Z, rmJ;
  struct FILEMAP *filemap;
  struct ELEMENT_ION_STATE *state=NULL; 

  *Atomic_params = (ElementP *) malloc_safe(N_Elements*sizeof(ElementP));

  load_filemap(filemap_file, &filemap);
  if (maxZ != NULL) *maxZ = 0;
  for (iZ=0; iZ < N_Elements; iZ++) {
    Z = list_Z[iZ];
    if (maxZ != NULL) if (Z > *maxZ) *maxZ = Z;
    state = get_state(Z);  /* Get some storage */
    
    for (irmJ=0; irmJ<list_NrmJ[iZ]; irmJ++) {
      rmJ = (list_rmJ[iZ])[irmJ];
      read_ion_data(require_checksum, Z, rmJ, filemap, maxNlev, 
		    &(state[rmJ-1]));
    }
    (*Atomic_params)[iZ] = state;  
  }
}

struct ELEMENT_ION_STATE * get_state(int Z) {

  struct ELEMENT_ION_STATE *state; 
  int rmJ;

  state = (struct ELEMENT_ION_STATE *)
    malloc_safe((Z+1)*sizeof(struct ELEMENT_ION_STATE));
  
  for (rmJ=1;rmJ<Z+2;rmJ++) {
    state[rmJ-1].ion_inf = (struct ION_INFORMATION *)
      malloc_safe(sizeof(struct ION_INFORMATION));
    
    state[rmJ-1].ion_inf->Z = Z;
    state[rmJ-1].ion_inf->rmJ = rmJ;
    sprintf(state[rmJ-1].ion_inf->Ion, "%s", roman_element(Z,rmJ-1));
    state[rmJ-1].ion_inf->nionrec = 0;    
    state[rmJ-1].ion_inf->nlev = 0;  
    state[rmJ-1].ion_inf->nwv = 0;   
    state[rmJ-1].ion_inf->nexc = 0;  
    state[rmJ-1].ion_inf->npro = 0;  
    state[rmJ-1].ion_inf->nsatel = 0;
    state[rmJ-1].ion_inf->nemiss = 0;
    
    state[rmJ-1].ionrec_dat = NULL;
    state[rmJ-1].lev_dat = NULL;
    state[rmJ-1].linerad_dat = NULL;
    state[rmJ-1].colltrns_dat = NULL;
    state[rmJ-1].protrns_dat = NULL;
    state[rmJ-1].satellite_dat = NULL;
    state[rmJ-1].emission_dat = NULL;
  }

  return state;
}
