FUNCTION calc_omega, Z, rmJ, ll, lu, E, SILENT=silent, FILE=file
;
;  Z is the element, rmJ the roman number of the ion, ll the lower
;  level, lu the upper level, and T the temperature, which may be a
;  vector.
;
;common Elev_com,  Z_Elev, rmJ_Elev, data_Elev, header_Elev
;common Ecoll_com, Z_Ecoll, rmJ_Ecoll, data_Ecoll, header_Ecoll
if (n_params() ne 5) then begin
    print,'Call as:'
    print,'ups=calc_omega(Z, rmJ, ll, lu, E, [/SILENT], [FILE=file])'
    print,'where Z is the element (proton) number,'
    print,'rmJ is the roman numeral of the ion state'
    print,'ll is the lower level of the transition'
    print,'lu is the upper level of the transition'
    print,'E is the energy, in keV (either vector or scalar)'
    return,0
endif

multi = 0   ; single transition only
KB = 8.617e-8
E_lu = calc_energy(Z,rmJ,lu)
E_ll = calc_energy(Z,rmJ,ll)
if ((E_lu lt 0.0)or(E_ll lt 0.0)) then begin
    if (n_elements(silent) eq 0) then print,'No appropriate energy data found'
    return,0*E
endif
dE = E_lu - E_ll
N_E = n_elements(E)
;
; Load collision strength structures
;
read_Ecoll, Z, rmJ, data, header, found=found, FILE=file
if (found ne 1) then begin
    if (n_elements(silent) eq 0) then print,'No appropriate data found'
    return,0*T
endif
;
; Now, does the appropriate ll, lu exist in this data structure?
;
Indx = where((data.lower_lev eq ll)and(data.upper_lev eq lu),count)
if (count eq 0) then begin
    if (n_elements(silent) eq 0) then $
      print,strcompress('No such transition '+string(ll)+' ->'+string(lu)+$
                        ' in the data structure')
    return,0*E
endif
if (count gt 1) then begin
;
;  This could be because the two transitions rates should be summed,
;  or because the transition type is interpolation and the multiple
;  transitions handle different temperature values.  None the less,
;  we'll print a warning at least.
;
    if (n_elements(silent) eq 0) then $
      print,strcompress('More than one such transition '+string(ll)+' ->'+$
                        string(lu)+' in the data structure.')
    multi = 1  ; multiple transitions.
endif

Ntr = n_elements(Indx)
result = fltarr(Ntr,N_E)

for iTr=0,Ntr-1 do begin
;
; OK, at this point, we've got a single good transition
;
    tr = data(Indx(iTr))
    if ((tr.Coeff_Type gt 30)and(tr.Coeff_Type lt 34)) then $ ; SGC
      result(iTr) = calc_sgc_om(tr, Z, E)

    if ((tr.Coeff_Type gt 40)and(tr.Coeff_Type lt 42)) then $ ; Kato
      result(iTr) = calc_kato_om(tr, E)
    
    if (tr.Coeff_Type eq 1) then begin ; Burgess&Tully
        print,'Not implemented yet.'
        stop
    endif
endfor

final = fltarr(N_E)
for iT=0,N_E-1 do final(iT) = total(result(*,iT))

return, final
end
