FUNCTION calc_spectrum_coco, coco_all, Ebin, T, ANGSTROM=angstrom, $
                       SILENT=silent,ELEMENTS=Zarr,ERGS=ergs
ERGperKEV = 1.60219e-9
;
;  Ebin is in keV, or in Angstroms.  Result is photons cm^3 / s / bin,
;  unless /ERGS is set in which case it's in ergs cm^3 / s / bin
;  Ebin must be in monotonically _INCREASING_ order.
;
;  Use when line is null!
;
if (n_params() lt 3) then begin
    print,'Calculates an emission spectrum in photons cm^3 / s / bin'
    print,'Call as:'
    print,'result = calc_spectrum_coco(coco, Ebin, T)'
    print,'where'
    print,'coco is an array of  compressed continuum structures, from read_coco'
    print,'Ebin is a user-defined energy (in keV) or wavelength (in Angstroms) grid'
    print,'T is a temperature, in K'
    print,' '
    print,'Optional Parameters:'
    print,'/ANGSTROM  : Set if passing a wavelength grid'
    print,'/SILENT    : Set to suppress any output except extreme errors'
    print,'ELEMENTS=Z : Z is an array of Z values to use in calculating spectrum'
    print,'/ERGS      : Set if you want the output in ergs cm^3 / s / bin'
    return,0.
endif
;
; First step is to extract the temperature; right now, we'll limit ourselves
; to temperatures actually calculated.
;
Tvec = coco_all.temperature
gp = where(abs(T - Tvec) eq min(abs(T - Tvec)))
; 
if ((min(abs(T-Tvec)) gt 0)and(n_elements(silent) eq 0)) then $
    print,'New temperature is ',coco_all(gp(0)).temperature
T = coco_all(gp(0)).temperature
;
; Now, restrict outselves to just this one temperature
;
count_coco = 0
if (n_elements(coco_all) ne 0) then $
  gp_coco = where(coco_all.temperature eq T,count_coco)
if (count_coco eq 0) then begin
    print,'Weird'
    stop
endif

NZ_coco = 0
if (count_coco ne 0) then begin
    coco = reform(coco_all(gp_coco))
    NZ_coco = n_elements(coco)
endif

count_coco = 0

if (n_elements(Zarr) ne 0) then begin
    match_coco = -1
    NZ_coco = 0
    for iZ=0,n_elements(Zarr)-1 do begin

        gp_coco = where(coco.Z eq Zarr(iZ),count_coco)

        if (count_coco gt 1) then begin
            print,'Strange'
            stop
        endif
        if (count_coco eq 1) then begin 
            NZ_coco = NZ_coco+1
            match_coco = [match_coco,gp_coco]
        endif
    endfor
    if (n_elements(match_coco) eq 1) then begin
        if (n_elements(silent) eq 0) then print,'No matching elements found'
    endif else begin
        coco = coco(match_coco(1:*))
    endelse
endif
;
;  Now make the spectrum
;
if (n_elements(angstrom) ne 0) then begin   ; Angstroms!
    Abin = Ebin
    Ebin = 12.398/Abin ; convert to keV
    indx = sort(Ebin)
    Ebin = Ebin(indx)
endif

Nbin = n_elements(Ebin)
Spectrum = fltarr(Nbin-1)

for iBin=0L,nbin-2 do begin
    for iZ=0,NZ_coco-1 do begin
        Continuum = expand_E_grid(Ebin(iBin),Ebin(iBin+1),coco(iZ).N_cont,$
                                  coco(iZ).E_cont, coco(iZ).continuum)
        Pseudo = expand_E_grid(Ebin(iBin),Ebin(iBin+1),coco(iZ).N_pseudo,$
                               coco(iZ).E_pseudo, coco(iZ).pseudo)
        if (n_elements(ergs) eq 0) then begin
            Spectrum(iBin) = Spectrum(iBin) + Continuum + Pseudo
        endif else begin
            Spectrum(iBin) = Spectrum(iBin) + $
              (Continuum + Pseudo)*ERGperKEV*0.5*(Ebin(iBin)+Ebin(iBin+1))
        endelse
    endfor
endfor

if (n_elements(angstrom) ne 0) then begin
    Ebin = Abin
    Spectrum = Spectrum(indx)  ; reverse spectrum to match angstroms
endif

return, Spectrum

end
