FUNCTION interp_header, header, keyword, INT_RESULT=integ

findstr = where(strpos(header,keyword) eq 0,count)
if (count gt 1) then begin
    print,'More than one', keyword, ' in the extension.'
    if (n_elements(integ) eq 0) then begin
        value = -1.0
    endif else begin
        value = -1
    endelse
endif

if (count eq 0) then begin
    print,'No ',keyword,' in the extension'
    if (n_elements(integ) eq 0) then begin
        value = -1.0
    endif else begin
        value = -1
    endelse
endif

if (count eq 1) then begin
    keyword_str = header(findstr)
    if (n_elements(integ) eq 0) then begin
        value = float(strmid(keyword_str,10,100))
    endif else begin
        value = fix(strmid(keyword_str,10,100))
    endelse
endif

return,value

end

PRO read_continuum, filename, continuum, T_vec, N_vec, lambda_vec
;
;  Reads the continuum data from a fits file
;
done = 0
ext = 1
c_var = {cont_S, Ebin_high: 1., Line:1., Line_Err:1., Pseudo:1.,$
         Pseudo_Err:1., Brems:1., Brems_Err:1., Two_Photon:1.,$
         TwPh_Err:1., Rad_Rec:1., Rad_Rec_Err:1.,temperature:1.,$
         density:1.}

while (not(done)) do begin
    data = mrdfits(filename,ext,header)
    if (n_tags(data) ne 0) then begin
        lambda_val = interp_header(header, 'TOT_EMIS')
        T_val = interp_header(header,'TEMPERAT')
        N_val = interp_header(header,'DENSITY')
        Nbin  = interp_header(header,'N_BIN',/INT_RESULT)
        if (ext eq 1) then begin
            T_vec = T_val
            N_vec = N_val
            lambda_vec = lambda_val
            continuum = replicate({cont_S},Nbin(0))
            continuum.Ebin_high = data.Ebin_high
            continuum.Line = data.Line
            continuum.Line_Err = data.Line_Err
            continuum.Pseudo = data.Pseudo
            continuum.Pseudo_Err = data.Pseudo_Err
            continuum.Brems = data.Brems
            continuum.Brems_Err = data.Brems_Err
            continuum.Two_Photon = data.Two_Photon
            continuum.TwPh_Err = data.TwPh_Err
            continuum.Rad_Rec = data.Rad_Rec
            continuum.Rad_Rec_Err = data.Rad_Rec_Err
            continuum.temperature = T_val(0)*(1.+fltarr(Nbin(0)))
            continuum.density = N_val(0)*(1.+fltarr(Nbin(0)))    
        endif else begin
            T_vec = [T_vec,T_val]
            N_vec = [N_vec,N_val]
            lambda_vec = [lambda_vec,lambda_val]
            cont_tmp = replicate({cont_S},Nbin(0))
            cont_tmp.Ebin_high = data.Ebin_high                   
            cont_tmp.Line = data.Line                             
            cont_tmp.Line_Err = data.Line_Err                     
            cont_tmp.Pseudo = data.Pseudo                         
            cont_tmp.Pseudo_Err = data.Pseudo_Err                 
            cont_tmp.Brems = data.Brems                           
            cont_tmp.Brems_Err = data.Brems_Err                   
            cont_tmp.Two_Photon = data.Two_Photon                 
            cont_tmp.TwPh_Err = data.TwPh_Err                   
            cont_tmp.Rad_Rec = data.Rad_Rec                       
            cont_tmp.Rad_Rec_Err = data.Rad_Rec_Err                 
            cont_tmp.temperature = T_val(0)*(1.+fltarr(Nbin(0)))
            cont_tmp.density = N_val(0)*(1.+fltarr(Nbin(0)))
            continuum = [[continuum],[cont_tmp]]
        endelse
        
        ext = ext + 1
    endif else begin
        done = 1
    endelse
endwhile

;continuum = transpose(continuum)

return
end


PRO read_linelist, filename, linelist, T_vec, N_vec, lambda_vec
;
;  Reads the linelist from a fits file.
;
done = 0
ext = 1
ll_var = {line_S, lambda: 1., epsilon:1., element:1, ion:1, temperature:1.,$
          density:1.}
header = headfits(filename,exten=ext)
while (n_elements(header) ne 1) do begin
    Nlines = interp_header(header,'N_LINES',/INT_RESULT)
    Nlines = Nlines(0)
    if (Nlines gt 0L) then begin    
        data = mrdfits(filename,ext,header)
        if (n_tags(data) ne 0) then begin
            lambda_val = interp_header(header, 'TOT_EMIS')
            T_val = interp_header(header,'TEMPERAT')
            N_val = interp_header(header,'DENSITY')
            if (n_elements(T_vec) eq 0) then begin
                T_vec = T_val
                N_vec = N_val
                lambda_vec = lambda_val
                linelist = replicate({line_S},Nlines(0))
                linelist.lambda = data.lambda
                linelist.epsilon= data.epsilon
                linelist.element= data.element
                linelist.ion = data.ion
                linelist.temperature = T_val(0)*(1.+fltarr(Nlines(0)))
                linelist.density = N_val(0)*(1.+fltarr(Nlines(0)))
            endif else begin
                T_vec = [T_vec,T_val]
                N_vec = [N_vec,N_val]
                lambda_vec = [lambda_vec,lambda_val]
                line_tmp = replicate({line_S},Nlines(0))
                line_tmp.lambda = data.lambda
                line_tmp.epsilon= data.epsilon
                line_tmp.element= data.element
                line_tmp.ion = data.ion
                line_tmp.temperature = T_val(0)*(1.+fltarr(Nlines(0)))
                line_tmp.density = N_val(0)*(1.+fltarr(Nlines(0)))
                linelist = [linelist,line_tmp]
            endelse
        endif
    endif
    ext = ext+1
    header = headfits(filename,exten=ext)
endwhile

return
end
