FUNCTION uncomp_spec, spectrum, Ebins, ZARR=Zarr, SILENT=silent

dE = Ebins(1:*) - Ebins     ; in keV
Eave = Ebins(1:*) - 0.5*dE  ; center of bins

if (n_elements(Zarr) eq 0) then begin
    gp = where((spectrum.Z eq 0)and(spectrum.rmJ eq 0))
endif else begin
    gp = where((spectrum.rmJ eq 0)) 
    spectmp = spectrum(gp)
    gp = [0]
    for iZ=0,n_elements(Zarr)-1 do begin
        gp1 = where(spectmp.Z eq Zarr(iZ),count)
        if (count gt 0) then gp = [gp,gp1]
    endfor
    if (n_elements(gp) gt 1) then gp = gp(1:*)
    if (n_elements(gp) le 1) then begin
        if (n_elements(silent) ne 0) then print,'No matching Z values'
        return,0*Ebins
    endif
endelse

result = 0.*Eave

if (n_tags(spectrum) eq 14) then begin ; This is a complete compressed struct
    for i=0,n_elements(gp)-1 do begin
        tmp1 = interpol(spectrum(gp(i)).brems,spectrum(gp(i)).e_brems, Eave)
        tmp1 = tmp1*dE          ; now in ph cm^3 /s /bin
        tmp2 =interpol(spectrum(gp(i)).two_photon,spectrum(gp(i)).e_twoph,Eave)
        tmp2 = tmp2*dE          ; now in ph cm^3 /s /bin
        tmp3 = interpol(spectrum(gp(i)).rad_rec,spectrum(gp(i)).e_RRC, Eave)
        tmp3 = tmp3*dE          ; now in ph cm^3 /s /bin
        result = result + tmp1 + tmp2 + tmp3
    endfor
endif
if (n_tags(spectrum) eq 10) then begin  ; This a atom-only comp cont file
    for i=0,n_elements(gp)-1 do begin
        N = spectrum(gp(i)).N_cont
        gp_E = where(Eave ge min(spectrum(gp(i)).e_cont(0:N-1)) and $
                     Eave le max(spectrum(gp(i)).e_cont(0:N-1)),count)
        if (count ne 0) then begin
            tmp = 0.*Eave
            tmp(gp_E) = interpol(spectrum(gp(i)).continuum(0:N-1),$
                                 spectrum(gp(i)).e_cont(0:N-1), Eave(gp_E))
            tmp = tmp*dE      ; now in ph cm^3 /s /bin
            result = result + tmp
        endif
    endfor
endif
if ((n_tags(spectrum) ne 14)and (n_tags(spectrum) ne 10)) then begin
    print,'Illegal structure passed.'
    stop
endif

return, result
end
