PRO write_ionbal
;
;  Set up the data variables
;
filename = 'PureO6_ionbal.fits'
logT0 = 4. ; First temperature, in log K   <--------
dT    = 5. ; delta T, in log K             <--------
N_T   = 2  ; Number of temperatures        <--------

n0    = 1.0; First density, in cm^-3       <--------
dn    = 1.0; delta n, in cm^-3             <--------
N_N   = 1  ; Number of densities           <--------

Z     = [8] ; Array of ions                <--------

logT = logT0 + dT*findgen(N_T)
density = n0 + dn*findgen(N_N)
nj    = Z+1 ; Array of number of Ion stages  
N_Z   = n_elements(Z)   ; Number of atoms    
N_I = fix(total(nj))    ; Number of ion stages
indx = fltarr(N_Z)       
for i=1,N_Z-1 do indx(i) = indx(i-1) + nj(i-1)  ; Index to ion stages

X = fltarr(N_T, N_N, N_I)  ; Ion population

X(0,0,5) = 1.0   ; Pure O VI
X(1,0,5) = 1.0   ; Pure O VI

MIN_X = 1.e-5

;
;  Now X(N_T, N_I) contains the entire ionization balance.  Z is the array
;  of elements, and logT holds all the temperatures
;
;  Time to make the file
;
;  Make the header
;
fxbhmake, ionbal_header, N_T,'ION_BAL', 'Ionization balance table',/DATE

fxaddpar, ionbal_header, 'T_FIRST' , logT0, '[K] First temperature'
fxaddpar, ionbal_header, 'T_DELTA' , dT,'[K] Delta temperature'
fxaddpar, ionbal_header, 'T_LINEAR',  'F','Linear (or log) temperature spacing'
fxaddpar, ionbal_header, 'T_NUMBER', N_T, 'Number of temperature grid points'
fxaddpar, ionbal_header, 'N_FIRST' , n0,  '[cm**(-3)] First density'
fxaddpar, ionbal_header, 'N_DELTA' , dn,  '[cm**(-3)] Delta density'
fxaddpar, ionbal_header, 'N_LINEAR',  'T','Linear (or log) density spacing'
fxaddpar, ionbal_header, 'N_NUMBER',  N_N,'Number of density grid points'
fxaddpar, ionbal_header, 'N_ELEMEN',  N_Z,'Number of elements'
fxaddpar, ionbal_header, 'N_IONS'  ,  N_I,'Total number of ion stages'
fxaddpar, ionbal_header, 'HDUCLASS', 'ION_BAL','Ionization balance tables'
fxaddpar, ionbal_header, 'HDUVERS1', '1.0.0','version of format'
;
;  Add the columns
;
fxbaddcol, index_T, ionbal_header, 1.0, 'Temperature', TUNIT='K'
fxbaddcol, index_N, ionbal_header, 1.0, 'Density', TUNIT='cm**-3'
fxbaddcol, index_Z, ionbal_header, Z, 'Z_ELEMENT'
fxbaddcol, index_X, ionbal_header, reform(X(0,0,*)), 'X_IONPOP'
;
;  Create the file and the binary table extension
;
fxhmake, primary_header, /extend
fxwrite, filename, primary_header
fxbcreate, unit, filename, ionbal_header
;
;  Write the data
;
zp = where(X le MIN_X,count)
if (count gt 0) then X(zp) = 0.0

for iT=0,N_T-1 do begin
    for iN=0,N_N-1 do begin
        iRow = iT*N_N + iN + 1
        fxbwrite, unit, 10^logT(iT), index_T, iRow
        fxbwrite, unit, density(iN), index_N, iRow
        fxbwrite, unit, Z, index_Z, iRow
        fxbwrite, unit, reform(X(iT,iN,*)), index_X, iRow
    endfor
endfor
;
;  Close the file
;
fxbfinish, unit

return
end
