%
%
%
%
% R. Hoogerwerf, SAO, 04/23/2002
%
%

uses("MATH-functions");

define AM_make_kernel();

define AM_broaden() {

   variable usage = "Usage: AM_broaden(source, fwhm, [\"pixel\"/\"angstrom\"])";

   variable src, fwhm, unit;
   variable isrc;

   if (0 == Utils->check_nargs(_NARGS,2,3)) {
      return;
   }
   
   if (_NARGS == 2) {
      (src, fwhm) = ();
      unit = "px";
   } else {
      (src, fwhm, unit) = ();
      if (unit == "pixel") {
	 unit = "px";
      } else if (unit == "angstrom") {
	 unit = "a";
      } else {
	 vmessage("AM_broaden: unit (%s) is not equal to \"pixel\" or \"angstrom\"; using \"pixel\".",unit);
	 unit = "px";
      }
   }
   
   if (String_Type == typeof(src)) {
      % src is a string. It might be of type dataset_range. Need to split it
      % and process.
      variable tmp = AM_parse_dataset_range(src);
      if (NULL != tmp) {
	 variable _isrc = Integer_Type[length(tmp)], i = 0;
	 foreach (tmp) {
	    _isrc[i] = (); i++;
	 }
      }
      isrc = _isrc;

   } else if (Integer_Type == typeof(src)) {
      % src is an integer. This makes this easy.
      isrc = [src];
   }

   %
   % Now set the correct fwhms in the broaden structure
   %
   foreach (isrc) {
      src = ();
      AM->broaden[src].num = src;
      AM->broaden[src].broaden = 1;
      AM->broaden[src].fwhm = fwhm;
      AM->broaden[src].unit = unit;
      AM->broaden[src].type = "gauss";   % default line profile
      AM->broaden[src].kernel = NULL;
   }      
}

define AM_kernel_type() {

   variable usage = "Usage: AM_kernel_type(source, type)";

   variable src, kernel;
   variable isrc;

   if (0 == Utils->check_nargs(_NARGS,2)) {
      return;
   }
   
   (src, kernel) = ();

   if (String_Type == typeof(src)) {
      % src is a string. It might be of type dataset_range. Need to split it
      % and process.
      variable tmp = AM_parse_dataset_range(src);
      if (NULL != tmp) {
	 variable _isrc = Integer_Type[length(tmp)], i = 0;
	 foreach (tmp) {
	    _isrc[i] = (); i++;
	 }
      }
      isrc = _isrc;

   } else if (Integer_Type == typeof(src)) {
      % src is an integer. This makes this easy.
      isrc = [src];
   }

   kernel = strlow(kernel);

   variable dummy;
   switch(kernel) 
   { case "gauss" : dummy = 1; }
   { case "lorentz" : dummy = 2; }
   { vmessage("Valid kernel types are [gauss,lorentz]; received %s.\nUsing Gaussian profile instead.",kernel); kernel = "gauss"; }

   %
   % Now set the correct fwhms in the broaden structure
   %
   foreach (isrc) {
      src = ();
      if (AM->broaden[src].num == NULL) {
	 vmessage("AM_broaden: No source model specified for %d using AM_broaden",src);
      } else {
	 AM->broaden[src].type = kernel;
      }
   }      
}


define AM_broaden_lines(flux,dataset) {

   variable kernel = AM->broaden[dataset].kernel;
   variable ii;


   if (AM->broaden[dataset].broaden == NULL) {
      ii =0;
      vmessage("%d",ii);
      return(flux);
   }

   if (AM->broaden[dataset].broaden == 1) {
   % this dataset has a broadening profile
      if (NULL == kernel) {
	 % see if fwhm is defined in pixels or angstroms. If A convert to
	 % pixels.
         if (AM->broaden[dataset].unit == "a") {
            AM->broaden[dataset].unit = "px";
            AM->broaden[dataset].fwhm = 
                (AM->broaden[dataset].fwhm/AM_get_data_binsize(dataset));
         }
	 %
	 AM->broaden[dataset].kernel = 
	     AM_make_kernel(AM->broaden[dataset].fwhm,
			    AM->broaden[dataset].type);
	 kernel = AM->broaden[dataset].kernel; 
      }
   }

   if (NULL == kernel) return flux; 

   variable npix = length(kernel);
   variable npix_extra = (npix - 1)/2;
   variable tmp = Double_Type[length(flux)+2*npix_extra];
   variable i,j;
   
   % initialize
   tmp[*] = 0.;
   
   for (i = 0; i < length(flux); i++) {
      for (j = -npix_extra; j <= npix_extra; j++) {
	 tmp[npix_extra + j + i] += kernel[npix_extra + j] * flux[i];
      }
   }
   
   return tmp[[npix_extra:length(flux)+npix_extra]];

}


define AM_make_kernel_gauss(fwhm) {

   %
   % Make a Gaussian kernel spanning 'fwhm' pixels over the 
   % full-width-at-half-maximum.
   %

   if (fwhm == 0.0) return NULL;

   variable i;
   variable fwhm_sigma = sqrt(log(4.0));
   variable fpix = 3. * fwhm;
   variable npix = Math->nint(fpix); if (Math->is_even(npix)) npix++;
   variable sig_min = -3. * fwhm_sigma * npix / fpix;
   variable sig_max =  3. * fwhm_sigma * npix / fpix;
   variable dsig    =  (sig_max - sig_min) / npix;
   
   variable kernel = Double_Type[npix];
   variable x1, x2;

   for (i = 0; i < npix; i++) {
      
      x1 = (sig_min + i * dsig);
      x2 = (sig_min + (i + 1) * dsig);
      kernel[i] = Math->Int_Gauss(x1,x2,0.0,1.0);


   }

   return kernel;

}

define AM_make_kernel_lorentz(fwhm) {

   %
   % Make a Lorentian kernel spanning 'fwhm' pixels over the 
   % full-width-at-half-maximum.
   %

   if (fwhm == 0.0) return NULL;

   variable i;
   variable fpix = 10. * fwhm;
   variable npix = Math->nint(fpix); if (Math->is_even(npix)) npix++;
   variable sig_min = -5. * fwhm * npix / fpix;
   variable sig_max =  5. * fwhm * npix / fpix;
   variable dsig    =  (sig_max - sig_min) / npix;

   variable kernel = Double_Type[npix];
   variable x1, x2;

   for (i = 0; i < npix; i++) {
      
      x1 = (sig_min + i * dsig);
      x2 = (sig_min + (i + 1) * dsig);
      kernel[i] = Math->Int_Lorentz(x1,x2,0.0,fwhm);

   }

   return kernel;

}

define AM_make_kernel(fwhm,type) {

   variable kernel;

   switch(type) 
   { case "gauss" : kernel = AM_make_kernel_gauss(fwhm); }
   { case "lorentz" : kernel = AM_make_kernel_lorentz(fwhm);}
   { vmessage("Valid kernel types are [gauss, lorentz]; received %s.\nUsing Gaussian profile instead.",type); kernel = AM_make_kernel_gauss(fwhm); }

   return kernel;

}

