%
% This file provides the actual model definition and calculation of 
% the model spectrum. This routine should tie together all input and
% free variables.
%
% Parts of this code have been taken from a simpler version of the 
% apec model written by Randall Smith (SAO).
%
% Ronnie Hoogerwerf, SAO, 04/10/2002
%

variable arfs;

if (__is_initialized(&arfs) == 0) {
   arfs = Struct_Type[100];
}

define read_arfs() {

   variable mdl, junk, arf, i=1;

   (mdl,junk) = get_mdl_data();
   
   foreach (mdl.arf) {
      arf = ();
      if (arf != "none") {
	 arfs[i] = readfile(arf);
      }
      i++;
   }

}

define rebin_flux(binlo, binhi, flux, lmin, lmax) {

   variable i1, i2;
   variable nbin = length(binlo);
   variable f;

   if (lmin > binhi[nbin-2]) {
      i1 = NULL;
   } else if (lmax < binlo[1]) {
      i1 = NULL;
   } else {
      i1 = where(binhi>=lmin);
      i1 = i1[0];
      i2 = i1+1;
    }

   %vmessage("%d %d %d",i1,i2,nbin);
   %vmessage("%f %f %f %f %f %f",lmin,lmax,binlo[0],binhi[0],binlo[nbin-1],binhi[nbin-1]);

   if (i1 == NULL) {
      f = 0.0;
   } else {
      f = flux[i1] * (binhi[i1] - lmin) / (binhi[i1] - binlo[i1]);
      f += flux[i2] * (lmax - binlo[i2]) / (binhi[i2] - binlo[i2]);
   }

   return f;

}

define linecenter() {  
   % linecenter
   
   variable i, nargs = 2 + 1;
   variable dataset, flux;
   variable inpbinlo, inpbinhi, index;
   variable lc;                                               %  1
   variable binlo, binhi;                                     %  2
                                                              %  3
   % first check if the routine is called with the correct number
   % of arguments.
   if ((_NARGS != nargs-1) and (_NARGS != nargs)) {
      message("Usage: LC->lc(linecenter,binlo,[binhi]");
      _pop_n(_NARGS);
      return NULL;
   }

   if (_NARGS == nargs-1) {
      %
      % This model is meant to run in the "integrate ON" so this is 
      % a cheap fix to change off to on
      %
      % get parameters from stack

      inpbinlo = ();
      %
      lc = ();

      % check if the wavelength arrays "binlo" and "binhi" are in
      % ascending order. If not fix it.
      if (length(inpbinlo) == 1) {
	 return [0];
      } else if (inpbinlo[1] < inpbinlo[0]) {
	 index = [length(inpbinlo)-1:0:-1];
      } else {
	 index = [0:length(inpbinlo)-1:1];
      }
      binlo = inpbinlo[index];
      binhi = @binlo;  % Copy array by dereferencing. Failing to do this
                       % will only copy the pointer and not the content.
      for(i = 0; i < length(binlo)-1; i++) binhi[i] = binlo[i+1];
      binhi[length(binlo)-1] = binlo[length(binlo)-1]+0.001;

   } else if (_NARGS == nargs) {
      % get parameters from stack

      inpbinhi = ();
      inpbinlo = ();
      %
      lc = ();

      % check if the wavelength arrays "binlo" and "binhi" are in
      % ascending order. If not fix it.
      if (length(inpbinlo) == 1) {
	 return [0];
      } else if (inpbinhi[0] < inpbinlo[0]) {
	 index = [length(inpbinlo)-1:0:-1];
      } else if (inpbinlo[1] < inpbinlo[0]) {
	 index = [length(inpbinlo)-1:0:-1];
      } else {
	 index = [0:length(inpbinlo)-1:1];
      }
      binlo = inpbinlo[index];
      binhi = inpbinhi[index];

   }
   % 
   dataset = get_dataset_number();
   %vmessage("%d %e", dataset, lc);

   variable data = get_data(dataset);
   variable j, n = length(data.y), nbinlo = length(binlo);
   variable tmp_flux;

   tmp_flux = Float_Type[nbinlo];
   for (i=0; i<n; i++) {
      for (j=0; j<nbinlo; j++) {
	 if (data.x0[i] > binlo[j] and data.x0[i] < binhi[j]) {
	    tmp_flux[j] += data.y[i];
	 }
      }
   }

   variable myarf=0;
   variable tmp_sflux = Float_Type[nbinlo];
   for (i=0; i<nbinlo; i++) {

      tmp_sflux[i] = rebin_flux(binlo,binhi,tmp_flux,
				2*lc-binhi[i],2*lc-binlo[i]);

      % also correct flux for arf and integration time
      myarf = arfs[dataset].SPECRESP[where(arfs[dataset].BIN_LO == binlo[i])];
      if (length(myarf) == 0) myarf = [1.0e10];
      tmp_sflux[i] /= (myarf[0]*data.exptime);

   }

   flux = tmp_sflux[index];

   %
   %
   % IO->write_block("flux.txt",binlo,binhi,tmp_flux);
   %
   %
   if (Float_Type == _typeof(inpbinlo)) {
      % message("Conversion to Float_Type");
      flux = typecast(flux, Float_Type);
   }

   return flux;

}



define make_lc() {

   % 
   % Use "sherpa_register_model" command to register the linecenter model with Sherpa
   % so that it will become available as a user model in sherpa and can be used
   % for fitting data.
   %

   if (is_defined("register_model") == 0) {
      error("You are using an old version of ciao. Please update to a more recent ciao version");
   }

   () = register_model("linecenter",["linecenter"],1,
		       [1.0],    % default values
		       [0.0],    % Min values
		       [100.0],  % Max values
		       [1]);     % Thaw or freeze

}

 
