define calc_weights(kT, kTarr) {
   %
   %  Assumes kTarr is sorted in increasing order
   %
   variable pos, weight, diff;
   
   if ((kT < min(kTarr)) or (kT > max(kTarr))) return NULL, NULL;

   variable gp = where(min(abs(kT - kTarr)) == abs(kT - kTarr));

   if (kT == kTarr[gp[0]]) return gp[0], 1.0;

   if (kTarr[gp[0]] < kT) {
      pos = [gp[0], gp[0]+1];
   } else {
      pos = [gp[0]-1, gp[0]];
   }
   diff = kTarr[pos[1]] - kTarr[pos[0]];
   weight = [(kTarr[pos[1]] - kT)/diff, (kT - kTarr[pos[0]])/diff];

   return pos, weight;
}

define apec() {  % T, abund, norm, binlo, binhi
   %
   %  Uses the newly-written plasma module
   %
   % flux = plasma_model (bin_lo, bin_hi); 
   %
   variable kT, abund, norm, binlo, binhi, inpbinlo, inpbinhi;
   variable linesuff, cocosuff, keV=-1;
   variable i, iAtom, index, result, gp, Z, savestack;
   variable pos_l, wt_l, kTarr_l, pos_c, wt_c, kTarr_c;
   variable lr=NULL,cr=NULL,l=NULL,c=NULL;
   variable l1,l2,c1,c2, c1_res, c2_res;
   
   if ((_NARGS != 6) and (_NARGS != 7)) {
     usage("wrong.");
     _pop_n(_NARGS);
     return NULL;
   }
   %  Are the X-axis units keV or Angstrom?
   if (_stkdepth() != 0) savestack = __pop_args(_stkdepth());
   variable dataset = get_dataset_number();
   variable curr_axis;
   if (dataset == -1) {
      curr_axis = NULL;
   } else {
      curr_axis = get_axes(dataset);
   }
   if (savestack != NULL) __push_args(savestack);
   
   if (length(curr_axis) != 1) {
      message("Error: dimension of dataset does not match that of model");
      return NULL;
   }
   if (curr_axis==NULL) {
      keV=1;
   } else {
      if (strcmp(curr_axis.axisunits,"keV")==0) keV=1; % keV units
      if (strcmp(curr_axis.axisunits,"A")==0)   keV=0; % Angstrom units
      if (keV == -1) {
	 message("Error: Input axis in neither keV nor Angstrom units.");
	 return NULL;
      }
   }
   %
   %  Get input variables
   %
   if (_NARGS == 6) {
      (kT, abund, linesuff, cocosuff, norm, inpbinlo)=();
      inpbinhi=NULL;
   } else {
      (kT, abund, linesuff, cocosuff, norm, inpbinlo, inpbinhi)=();
   }
   linesuff=int(linesuff);
   cocosuff=int(cocosuff);
   %
   %  Check to make sure the line, coco values are defined.
   %
   if (linesuff >= 0) lr = __get_reference(sprintf("_ciao->line%d",linesuff));
   if (cocosuff >= 0) cr = __get_reference(sprintf("_ciao->cont%d",cocosuff));
   if (((linesuff >= 0)and(lr == NULL)) or
       ((cocosuff >= 0)and(cr == NULL))) {
      if (lr==NULL) vmessage("Error: line array line%d not defined.",linesuff);
      if (cr==NULL) vmessage("Error: cont array cont%d not defined.",cocosuff);
      verror("Use (line%d, cont%d) = load_plasma() before calling.",
	     linesuff,cocosuff);
   }
   if ((lr != NULL)and(linesuff >= 0)) l=@lr;
   if ((cr != NULL)and(cocosuff >= 0)) c=@cr;
   %
   %  Now, loop over plasma model, once for each atom
   %
   variable res_line = Double_Type [length(inpbinlo)];
   variable res_coco = Double_Type [length(inpbinlo)];
   result = Double_Type [length(inpbinlo)];
   for (i=0;i<length(inpbinlo);i++) {
      result[i] = 0.0;
      res_line[i] = 0.0;
      res_coco[i] = 0.0;
   }
   if (c!=NULL) {
      kTarr_c = c["0"].kT;
      (pos_c, wt_c) = calc_weights(kT, kTarr_c);
      if (pos_c == NULL) verror("kT = %f out of range (%f-%f)",kT,min(kTarr_c),max(kTarr_c));
   }
   if (l!=NULL) {
      kTarr_l = l["0"].kT;
      (pos_l, wt_l) = calc_weights(kT, kTarr_l);
      if (pos_l == NULL) verror("kT = %f out of range (%f-%f)",kT,min(kTarr_l),max(kTarr_l));
   }   

   if (l!=NULL) {
      l1 = l[string(pos_l[0]+1)];  % Plus 1 due to offset in fits tables
      l2 = l[string(pos_l[1]+1)];
      res_line = (wt_l[0]*plasma_line(1.0, l1.Lambda, l1.Epsilon,
				      keV, inpbinlo, inpbinhi) +
		  wt_l[1]*plasma_line(1.0, l2.Lambda, l2.Epsilon,
				      keV, inpbinlo, inpbinhi));
   }

   if (c!=NULL) {
      c1 = c[string(pos_c[0]+1)];
      c2 = c[string(pos_c[1]+1)];
      
      for (iAtom=0; iAtom<c["0"].NElement[pos_c[0]]; iAtom++) {
	 res_coco = res_coco +
	   wt_c[0]*plasma_coco(1.0,c1.N_Cont[iAtom],c1.E_Cont[iAtom,*],
			       c1.Continuum[iAtom,*],c1.N_Pseudo[iAtom],
			       c1.E_Pseudo[iAtom,*],c1.Pseudo[iAtom,*],
			       keV, inpbinlo, inpbinhi) + 
	   wt_c[1]*plasma_coco(1.0,c2.N_Cont[iAtom],c2.E_Cont[iAtom,*],
			       c2.Continuum[iAtom,*],c2.N_Pseudo[iAtom],
			       c2.E_Pseudo[iAtom,*],c2.Pseudo[iAtom,*],
			       keV, inpbinlo, inpbinhi);
      }
   }
   result = abund*norm*1.e14*(res_line + res_coco);
   if (Float_Type == _typeof(inpbinlo)) result = typecast(result, Float_Type);
   return result;
}

() = register_model("apec",
		    ["kT","abund","ldex","cdex","EM"],  % parameters
		    1,                       % 1 dimensional
                    [1.0 , 1.0, 1, 1,1.0], % default values
                    [0.01, 0.0,-1,-1,0.0 ], % Minimum values
                    [80. ,100., 1, 1,1.e10], % Maximum values
                    [1   ,   0, 0, 0,1]);    % Float temp and norm
