#include <math.h>
#include <stdlib.h>
#include <strings.h>
#include <stdio.h>
#include "descript.h"
#include "messages.h"
#define TRUE 1
#define FALSE 0
#define ANDERSGREVESSE 1
#define DEPLETED 2
#define ALLEN 3
#define GREVESSEANDERS 4
#define ISOTHERMAL 0
#define ISOBARIC 1
#define ISOCHORIC 2
#define ISOSWITCH 3
#define NORMAL 0
#define EQUILIBRIUM 1
#define MAXLINES 2000
int strip_strcmp(const char *s1, const char *s2);
int matchstr(const char *s1, const char *s2, int *m);
void read_parm_file(char *filename, struct descript **des);
void parse_atom(char *atomstring, int *atom, int *ion);
void disallow(struct descript *des);

#ifdef SOLARIS
void get_data_(char outfile[MAXSTRLEN], char wisfile[MAXSTRLEN],
	       int *rtype, int *abundtype,
	       int *dust, float *porosity, float *MaxTime, float *MaxFluenc,
	       float *dT, float *T_switch, float *Tmin,float *Telec,
	       float *Tion, float *pressure, int *wisconsin,
	       float Emis_lines[MAXLINES], int atom_lines[MAXLINES],
	       int ion_lines[MAXLINES], int *numlines, float *min_line_sep,
	       int outfilesize, int wisfilesize) {
#endif
#ifdef LINUX
void get_data_(char outfile[MAXSTRLEN], char wisfile[MAXSTRLEN],
               int *rtype, int *abundtype,
               int *dust, float *porosity, float *MaxTime, float *MaxFluenc,
               float *dT, float *T_switch, float *Tmin,float *Telec,
               float *Tion, float *pressure, int *wisconsin,
               float Emis_lines[MAXLINES], int atom_lines[MAXLINES],
               int ion_lines[MAXLINES], int *numlines, float *min_line_sep,
               int outfilesize, int wisfilesize) {
#endif

  int slen, m;
  char filename[MAXSTRLEN];
  struct descript *des;
  char *st_out = ".out";
  char *st_wis = ".wis";
  read_parm_file(NULL,&des);

  *filename = '\0';
  *rtype = 0;
  *abundtype = GREVESSEANDERS;
  *dust = FALSE;
  *porosity = 0.0;
  *MaxTime = 0.;
  *MaxFluenc = 0.;
  *dT = 0.02;
  *T_switch = 0.;
  *Tmin = 0.;
  *Telec = 0.;
  *Tion = 0.;
  *wisconsin = FALSE;
  *pressure = -1.;
  *min_line_sep = 1.0;  /* minimum distance from correct line, in Angstroms */
  *numlines = 0;
  
  while (des != NULL) {
    m = FALSE;
    if (matchstr(des->descriptor,"WISCONSIN",&m)) *wisconsin  = TRUE;
    if (matchstr(des->descriptor,"USEDUST",&m))   *dust = TRUE;
    if (matchstr(des->descriptor,"FPRESSURE",&m)) *pressure= des->value.fval;
    if (matchstr(des->descriptor,"FPOROSITY",&m)) *porosity = des->value.fval;
    if (matchstr(des->descriptor,"FMAXTIME",&m))  *MaxTime = des->value.fval;
    if (matchstr(des->descriptor,"FMAXFLUENCE",&m))*MaxFluenc=des->value.fval;
    if (matchstr(des->descriptor,"FT_ELEC",&m))   *Telec = des->value.fval; 
    if (matchstr(des->descriptor,"FT_ION",&m))    *Tion = des->value.fval; 
    if (matchstr(des->descriptor,"FDELTA_T",&m))  *dT = des->value.fval; 
    if (matchstr(des->descriptor,"FT_SWITCH",&m)) *T_switch = des->value.fval;
    if (matchstr(des->descriptor,"FT_MIN",&m))    *Tmin = des->value.fval;
    if (matchstr(des->descriptor,"FMINSEP",&m))*min_line_sep = des->value.fval;
    if (matchstr(des->descriptor,"SATOM",&m)) {
      parse_atom(des->value.sval, &(atom_lines[*numlines]),
		 &(ion_lines[*numlines]));
    }
    if (matchstr(des->descriptor,"FLINE",&m)) {
      Emis_lines[*numlines] = des->value.fval;
      (*numlines)++;
    }
    if (matchstr(des->descriptor,"STYPE",&m))  {
      if (strip_strcmp(des->value.sval,"ISOTHERMAL")==0)  *rtype = ISOTHERMAL;
      if (strip_strcmp(des->value.sval,"ISOBARIC")==0)    *rtype = ISOBARIC;
      if (strip_strcmp(des->value.sval,"ISOCHORIC")==0)   *rtype = ISOCHORIC;
      if (strip_strcmp(des->value.sval,"ISOSWITCH")==0)   *rtype = ISOSWITCH;
    }
    if (matchstr(des->descriptor,"SABUNDTYPE",&m)) {
      if (strip_strcmp(des->value.sval,"Depleted") == 0) 
	{ *abundtype = DEPLETED; } 
      else if (strip_strcmp(des->value.sval,"AndersGrevesse") == 0)
	{ *abundtype = ANDERSGREVESSE; }
      else if (strip_strcmp(des->value.sval,"Allen") == 0)
	{ *abundtype = ALLEN; }
      else if (strip_strcmp(des->value.sval,"GrevesseAnders") == 0)
	{ *abundtype = GREVESSEANDERS; }
      else {
	message("get_data","The string");
	mess_str(des->value.sval);
	errmess("get_data","does not match any allowed abundance types.");
      }
    }
    if (matchstr(des->descriptor,"SOUTFILE",&m)) {
      slen = strlen(des->value.sval);
      if (slen > MAXSTRLEN-10) {
        errmess("get_data","Output filename too long");
      }
      strcpy(filename,des->value.sval);
    }
    des = des->next;
  }
  if (*filename != '\0') {
    strcpy(outfile,filename);
    strcpy(wisfile,filename);
    strcat(outfile,st_out);
    strcat(wisfile,st_wis);
  } else {
    errmess("get_data","No output file (SOUTPUT) set.");
  }
}

void disallow(struct descript *des) {

  message("disallow","The string");
  mess_str(des->descriptor);
  mess_str(" is not currently allowed.  Reprogram if it is needed.\n");
  errmess("disallow","Bailing out.");
}

int matchstr(const char *s1, const char *s2, int *m) {
  
  int result=FALSE;
  
  if (strcmp(s1,s2) == 0) {
    result = TRUE;
    if (!(*m)) {
      *m = TRUE;
    } else {
      message("matchstr","Matched on two strings without clearing first.");
      message("matchstr","Strings were ");
      message("matchstr",s1);
      message("matchstr",s2);
    }
  }
  return result;
}


int strip_strcmp(const char *s1, const char *s2) {

  /* Strips any whitespace from string s1, and compares it to s2. */

  char strip_string[MAXSTRLEN];

/*  strcpy(strip_string,s1); */
  sscanf(s1,"%s",strip_string);
  return strcmp(strip_string,s2);
}

