C Author: Dick Edgar
C Date: 4/13/92
C Version: 2.1
C Language: Fortran 77
C Routine: isocool
C Invocation: isocool < inputfile
C Purpose: A program to calculate partial cooling per element for
C a suddenly increased temperature.
C Revisions: (RKS = Randall K Smith)
c
c     9/19/97 RKS Modified heavily again to fit into hydrodynamic utility
c     format.  Also, now calculates results using RS, so as to get Oxygen
c     lines directly.
c
c     11/18/93 RKS Modified heavily to use new input routine, and to
c     automatically allow for spectra and column density calculations.
c
c     10/26/93 RKS Modified to allow for various porosities of dust
c                  grains.  
c
c     2/16/93 RKS Modified to run isobaric, isochoric, and isothermal
c                 cases all in one.  Density fluctuation problem fixed.
c
c     10/29/93 RKS Modified to account for porosity of silicate dust grains.
c
C Side Effects: Creates an output file, with name chosen to indicate the 
C inputs.
C Other Notes: Earlier version used for the 1st sputtering paper, 
c     Time-Dependent Sputtering, Smith etal
C End:

      program cooling

      implicit none
      integer NUM2,MAXLINES
      parameter(NUM2=14,MAXLINES=2000)


      include 'jr_common.inc'

      character*128 OutFile, WisFile
      real Tion, Telec, Tend, Tsw, dT
      integer AbundType,CoolingType, Wisconsin
      real Porosity, MaxTime, Pressure, MaxFluenc
      integer usedust,indx(NUM)
c
c     Variables for the line emission code
c      
      integer totnumlines, atom_indexfull(MAXLINES),
     $     ion_indexfull(MAXLINES)
      integer numlines, atom_index(MAXLINES), ion_index(MAXLINES)
      integer line_index(MAXLINES)
      real emis_lines(MAXLINES), min_line_sep
      integer atom_lines(MAXLINES), ion_lines(MAXLINES)
      character*10 line_name(MAXLINES)
c
c     Elements are: He,C ,N ,O ,Ne,Mg,Si,S ,Ar,Ca,Fe,Ni
      data indx/1 ,1 ,1 ,1 ,1 ,1 ,1 ,1 ,1 ,1 ,1 ,1/
c
c Get the startup data
c
      call GetInitialData(OutFile, WisFile, CoolingType,Tion,Telec,
     $     Tsw, usedust,Porosity,MaxTime,MaxFluenc,AbundType,dT,Tend,
     $     Pressure, Wisconsin, Emis_lines, atom_lines, ion_lines,
     $     numlines, min_line_sep)
c
c Set up to use RS
c
      call setup_raymond(AbundType, indx)
      if (Wisconsin.eq.1) call setup_wisc(WisFile)
c
c     Read in line information from Raymond code
c     
      call read_linelist(atom_indexfull, ion_indexfull, line_name,
     $     totnumlines)
c
c     Check lines read in versus lines asked for.
c
      call find_lines(min_line_sep,atom_indexfull, ion_indexfull,
     $     line_name, totnumlines, emis_lines, atom_lines, ion_lines,
     $     numlines, atom_index, ion_index, line_index)
c
c Now, print out all the initial parameters.
c      
      call init_output(OutFile,CoolingType,Tion,Telec,Tsw, 
     $     usedust,Porosity,MaxTime,MaxFluenc,AbundType,dT,Tend,
     $     Pressure, Emis_lines, atom_lines, ion_lines, numlines)
c
c Get abundances
c
c      if (usedust.eq.1) call readdustinfo(porosity,abund)
c      if (AbundType.ne.2) then  ! Solar case.
c         do i=2,NUM
c            abund(i) = abunj(i-1)
c         end do
c         abund(1) = 12.0
c      end if
c
c Do run
c
      call calc_evolve(Telec, Tion, Tend, Tsw, dT, CoolingType,
     $     MaxTime, MaxFluenc, UseDust, Pressure, Wisconsin,
     $     numlines, atom_index, ion_index)
c
c Finish up.
c      
      call close_output(Wisconsin)
c      
      stop 'normal'
      end
