/** \file GuiOStream.cxx
    \brief Implementation of GuiOStream class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "TGTextView.h"

#include "profit/GuiOStream.h"

namespace profit {

  GuiOStream::GuiOStream(TGTextView * text_view): OStream(false), m_os(), m_text_view(text_view) {
    // Connect the string stream which is used for formatting the output for the text view.
    connect(m_os);
  }

  void GuiOStream::postProcess() {
    // Iterate over string contents, displaying only complete lines in the text view.
    std::string::size_type begin = 0;
    std::string::size_type end = m_os.str().find_first_of("\n");
    for (; std::string::npos != end; end = m_os.str().find_first_of("\n", begin)) {
      // Add the current token to the text view.
      m_text_view->AddLine(m_os.str().substr(begin, end).c_str());
      // Start looking for the next token immediately after the newline which terminated this string.
      begin = end + 1;
    }
    // Remove everthing which was just displayed from the buffer.
    if (0 != begin) m_os.str(m_os.str().substr(begin));

    // Scroll down.
    // TODO: This hack is to prevent going to the "bottom" when no scrollbar is displayed, which
    // has the effect of making the text invisible.
    int line_count = m_text_view->ReturnLineCount();
    if (line_count > 6) m_text_view->SetVsbPosition(line_count);
  }

}
